/*** Autogenerated by WIDL 11.16 from ../wine/include/ndattrib.idl - Do not edit ***/

#ifdef _WIN32
#ifndef __REQUIRED_RPCNDR_H_VERSION__
#define __REQUIRED_RPCNDR_H_VERSION__ 475
#endif
#include <rpc.h>
#include <rpcndr.h>
#endif

#ifndef COM_NO_WINDOWS_H
#include <windows.h>
#include <ole2.h>
#endif

#ifndef __ndattrib_h__
#define __ndattrib_h__

/* Forward declarations */

/* Headers for imported files */

#include <unknwn.h>
#include <objidl.h>
#include <oaidl.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum __WIDL_ndattrib_generated_name_0000000E {
    UIT_INVALID = 0,
    UIT_NONE = 1,
    UIT_SHELL_COMMAND = 2,
    UIT_HELP_PANE = 3,
    UIT_DUI = 4
} UI_INFO_TYPE;
typedef enum tagREPAIR_SCOPE {
    RS_SYSTEM = 0,
    RS_USER = 1,
    RS_APPLICATION = 2,
    RS_PROCESS = 3
} REPAIR_SCOPE;
#define PREPAIR_SCOPE  *REPAIR_SCOPE
typedef enum __WIDL_ndattrib_generated_name_0000000F {
    RR_NOROLLBACK = 0,
    RR_ROLLBACK = 1,
    RR_NORISK = 2
} REPAIR_RISK;
typedef struct tagShellCommandInfo {
    LPWSTR pwszOperation;
    LPWSTR pwszFile;
    LPWSTR pwszParameters;
    LPWSTR pwszDirectory;
    ULONG nShowCmd;
} ShellCommandInfo;
typedef struct tagShellCommandInfo *PShellCommandInfo;
typedef struct tagUiInfo {
    UI_INFO_TYPE type;
    __C89_NAMELESS union {
        LPWSTR pwzNull;
        ShellCommandInfo ShellInfo;
        LPWSTR pwzHelpUrl;
        LPWSTR pwzDui;
    } __C89_NAMELESSUNIONNAME;
} UiInfo;
typedef struct tagUiInfo *PUiInfo;
typedef struct tagRepairInfo {
    GUID guid;
    LPWSTR pwszClassName;
    LPWSTR pwszDescription;
    DWORD sidType;
    LONG cost;
    ULONG flags;
    REPAIR_SCOPE scope;
    REPAIR_RISK risk;
    UiInfo UiInfo;
    int rootCauseIndex;
} RepairInfo;
typedef struct tagRepairInfo *PRepairInfo;
typedef struct tagRepairInfoEx {
    RepairInfo repair;
    USHORT repairRank;
} RepairInfoEx;
typedef struct tagRepairInfoEx *PRepairInfoEx;
typedef struct tagRootCauseInfo {
    LPWSTR pwszDescription;
    GUID rootCauseID;
    DWORD rootCauseFlags;
    GUID networkInterfaceID;
    RepairInfoEx *pRepairs;
    USHORT repairCount;
} RootCauseInfo;
typedef struct tagRootCauseInfo *PRootCauseInfo;
/* Begin additional prototypes for all interfaces */


/* End additional prototypes */

#ifdef __cplusplus
}
#endif

#endif /* __ndattrib_h__ */
