/*
 * Copyright 2019 Alistair Leslie-Hughes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */

import "unknwn.idl";
import "objidl.idl";
import "oaidl.idl";
import "msinkaut.idl";

cpp_quote("#define TABLET_DISABLE_PRESSANDHOLD        0x00000001")

cpp_quote("#define WM_TABLET_DEFBASE                  0x02c0")
cpp_quote("#define WM_TABLET_MAXOFFSET                0x20")
cpp_quote("#define WM_TABLET_ADDED                    (WM_TABLET_DEFBASE + 8)" )
cpp_quote("#define WM_TABLET_DELETED                  (WM_TABLET_DEFBASE + 9)" )
cpp_quote("#define WM_TABLET_FLICK                    (WM_TABLET_DEFBASE + 11)" )
cpp_quote("#define WM_TABLET_QUERYSYSTEMGESTURESTATUS (WM_TABLET_DEFBASE + 12)" )

typedef USHORT SYSTEM_EVENT;
typedef DWORD TABLET_CONTEXT_ID;

typedef enum _PROPERTY_UNITS
{
    PROPERTY_UNITS_DEFAULT     = 0,
    PROPERTY_UNITS_INCHES      = 1,
    PROPERTY_UNITS_CENTIMETERS = 2,
    PROPERTY_UNITS_DEGREES     = 3,
    PROPERTY_UNITS_RADIANS     = 4,
    PROPERTY_UNITS_SECONDS     = 5,
    PROPERTY_UNITS_POUNDS      = 6,
    PROPERTY_UNITS_GRAMS       = 7,
    PROPERTY_UNITS_SILINEAR    = 8,
    PROPERTY_UNITS_SIROTATION  = 9,
    PROPERTY_UNITS_ENGLINEAR   = 10,
    PROPERTY_UNITS_ENGROTATION = 11,
    PROPERTY_UNITS_SLUGS       = 12,
    PROPERTY_UNITS_KELVIN      = 13,
    PROPERTY_UNITS_FAHRENHEIT  = 14,
    PROPERTY_UNITS_AMPERE      = 15,
    PROPERTY_UNITS_CANDELA     = 16
} PROPERTY_UNITS;

typedef struct _PROPERTY_METRICS
{
    LONG nLogicalMin;
    LONG nLogicalMax;
    PROPERTY_UNITS Units;
    FLOAT fResolution;
} PROPERTY_METRICS;

typedef struct _PROPERTY_METRICS *PPROPERTY_METRICS;

typedef struct _PACKET_PROPERTY
{
    GUID guid;
    PROPERTY_METRICS PropertyMetrics;
} PACKET_PROPERTY;

typedef struct _PACKET_PROPERTY *PPACKET_PROPERTY;

typedef struct tagSYSTEM_EVENT_DATA
{
    BYTE bModifier;
    WCHAR wKey;
    LONG xPos;
    LONG yPos;
    BYTE bCursorMode;
    DWORD dwButtonState;
} SYSTEM_EVENT_DATA;
